"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _path = _interopRequireDefault(require("path"));

var _url = require("url");

var _querystring = _interopRequireDefault(require("querystring"));

var _mem = _interopRequireDefault(require("mem"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function getPaths(stats, options) {
  const childStats = stats.stats ? stats.stats : [stats];
  const publicPaths = [];

  for (const {
    compilation
  } of childStats) {
    // The `output.path` is always present and always absolute
    const outputPath = compilation.getPath(compilation.outputOptions.path);
    const publicPath = options.publicPath ? compilation.getPath(options.publicPath) : compilation.outputOptions.publicPath ? compilation.getPath(compilation.outputOptions.publicPath) : '';
    publicPaths.push({
      outputPath,
      publicPath
    });
  }

  return publicPaths;
}

const memoizedParse = (0, _mem.default)(_url.parse);

var _default = (context, url) => {
  const {
    options
  } = context;
  const paths = getPaths(context.stats, options);
  let filename;
  let urlObject;

  try {
    // The `url` property of the `request` is contains only  `pathname`, `search` and `hash`
    urlObject = memoizedParse(url, false, true);
  } catch (_ignoreError) {
    return filename;
  }

  for (const {
    publicPath,
    outputPath
  } of paths) {
    let publicPathObject;

    try {
      publicPathObject = memoizedParse(publicPath || '/', false, true);
    } catch (_ignoreError) {
      // eslint-disable-next-line no-continue
      continue;
    }

    if (urlObject.pathname && urlObject.pathname.startsWith(publicPathObject.pathname)) {
      filename = outputPath; // Strip the `pathname` property from the `publicPath` option from the start of requested url
      // `/complex/foo.js` => `foo.js`

      const pathname = urlObject.pathname.substr(publicPathObject.pathname.length);

      if (pathname) {
        filename = _path.default.join(outputPath, _querystring.default.unescape(pathname));
      }

      let fsStats;

      try {
        fsStats = context.outputFileSystem.statSync(filename);
      } catch (_ignoreError) {
        // eslint-disable-next-line no-continue
        continue;
      }

      if (fsStats.isFile()) {
        break;
      } else if (fsStats.isDirectory() && (typeof options.index === 'undefined' || options.index)) {
        const indexValue = typeof options.index === 'undefined' || typeof options.index === 'boolean' ? 'index.html' : options.index;
        filename = _path.default.join(filename, indexValue);

        try {
          fsStats = context.outputFileSystem.statSync(filename);
        } catch (__ignoreError) {
          // eslint-disable-next-line no-continue
          continue;
        }

        if (fsStats.isFile()) {
          break;
        }
      }
    }
  }

  return filename;
};

exports.default = _default;